%% Generate figure 1 - measurement cycles
%
% Author:           Benjamin Schuler
% Company:          University of Stuttgart, Institute for System Dynamics
% Last modified:    21.02.2019 (Matlab 2015a)

clear; close all; clc;
cfroot = cd;

%% settings

% window for wavenumbers
w_LB = 1000;        % lower bound for exclusion from ALSS
w_UB = 1100;        % upper bound for exclusion from ALSS
waveband = 3;      	% band of wavenumbers around characteristic wavenumbers nu_i

% characteristic wavenumbers
wn1 = 1034;    % [1/cm]    % 1. characteristic wavenumber of glucose
wn2 = 1063;    % [1/cm]    % 1. characteristic wavenumber of fructose
wn3 = 1078;    % [1/cm]    % 2. characteristic wavenumber of glucose
wn4 = 1080;    % [1/cm]    % 2. characteristic wavenumber of fructose

% Load plotstyle
load('plotstyleBS.mat')
fs        = 14;     % adjust font size
fs_legend = 13;     % font size in legend
mgrCD = (lgrCD+dgrCD)/2;    % additional color
clr1 = [repmat(mgrCD,6,1); lg; lg;...
       0.5*(lg+dg); 0.5*(lg+dg); dg]; % color vector
clr2 = [repmat(mgrCD,6,1); lbCD; lbCD;...
       0.5*(lbCD+dbCD); 0.5*(lbCD+dbCD); dbCD]; % color vector
clr3 = [repmat(mgrCD,6,1);...
        lg; 1/3*lg+2/3*lbCD; lbCD; 2/3*lg+1/3*lbCD; (lg+lbCD)/2]; % color vector
    
% Concentration levels (1 for water-vizualization)
cVec1 = [1,10,1,25,1,50,1,25,1,10,1]'; % for gc / fc
cVec2 = [1,50,1,0,1,50,1,60,1,30,1]';  % for mc (glucose)
cVec3 = [1,0,1,50,1,50,1,30,1,60,1]';  % for mc (fructose)

%%

% Figure
hf = figure('color','white','visible','off');

barWidth     = 0.45;
barWidth2    = 0.2;
barOffset    = 0.2;
txtOffsetW   = 5;
txtOffsetGF  = 4;
txtOffsetLR  = 0.2; %0.25

% glucose cycle
hs(1) = subplot(2,3,1);
hb1 = bar(1:2:11,cVec1(1:2:11),barWidth,'FaceColor',mgrCD,'EdgeColor',mgrCD); hold on;
hb2 = bar(2:2:10,cVec1(2:2:10),barWidth,'FaceColor',lg,'EdgeColor',lg);
hl1 = legend([hb1 hb2],{'water','glucose'},'location','north','orientation','horizontal');
for ii=1:2:11
    text(ii,cVec1(ii,1)-txtOffsetW,num2str(cVec1(ii,1)-1,'% 2.0f'),'Color',mgrCD,'HorizontalAlignment','center');
end
for ii=2:2:10
    text(ii,cVec1(ii,1)+txtOffsetGF,num2str(cVec1(ii,1),'% 2.0f'),'Color',lg,'HorizontalAlignment','center');
end

% fructose cycle
hs(2) = subplot(2,3,2);
hb1 = bar(1:2:11,cVec1(1:2:11),barWidth,'FaceColor',mgrCD,'EdgeColor',mgrCD); hold on;
hb2 = bar(2:2:10,cVec1(2:2:10),barWidth,'FaceColor',lbCD,'EdgeColor',lbCD);
hl2 = legend([hb1 hb2],{'water','fructose'},'location','north','orientation','horizontal');
for ii=1:2:11
    text(ii,cVec1(ii,1)-txtOffsetW,num2str(cVec1(ii,1)-1,'% 2.0f'),'Color',mgrCD,'HorizontalAlignment','center');
end
for ii=2:2:10
    text(ii,cVec1(ii,1)+txtOffsetGF,num2str(cVec1(ii,1),'% 2.0f'),'Color',lbCD,'HorizontalAlignment','center');
end

% mixed cycle
hs(3) = subplot(2,3,3);
hb1 = bar((1:2:11),cVec2(1:2:11),barWidth,'FaceColor',mgrCD,'EdgeColor',mgrCD); hold on;
hb3 = bar((2:2:10)-barOffset,[50,1,50,60,30],barWidth2,'FaceColor',lg,'EdgeColor',lg);
hb4 = bar((2:2:10)+barOffset,[1,50,50,30,60],barWidth2,'FaceColor',lbCD,'EdgeColor',lbCD);
for ii=1:2:11
    text(ii,cVec2(ii,1)-txtOffsetW,num2str(cVec3(ii,1)-1,'% 2.0f'),'Color',mgrCD,'HorizontalAlignment','center');
end
for ii=2:2:10
    if ii==6
        text(ii-barOffset,cVec2(ii,1)+txtOffsetGF*2,num2str(cVec2(ii,1),'% 2.0f'),'Color',lg,'HorizontalAlignment','center');
        text(ii+barOffset,cVec3(ii,1)+txtOffsetGF,num2str(cVec3(ii,1),'% 2.0f'),'Color',lbCD,'HorizontalAlignment','center');
    else
        text(ii-barOffset,cVec2(ii,1)+txtOffsetGF,num2str(cVec2(ii,1),'% 2.0f'),'Color',lg,'HorizontalAlignment','center');
        text(ii+barOffset,cVec3(ii,1)+txtOffsetGF,num2str(cVec3(ii,1),'% 2.0f'),'Color',lbCD,'HorizontalAlignment','center');
    end
end

% Adjust subplots
for ii=1:3
    subplot(2,3,ii);
    set(gca,'XTick',1:11);
%     set(gca,'XTickLabel',{});
    set(gca,'YTick',[0 10 20 30 40 50 60]);
    if ii==1
        ylabel('c [g/l]')
    else
        ylabel(' ');
        set(gca,'YTickLabel',{});
    end
    ylim([-10 70]); 
    xlim([0.25 11.75]);
    grid on; box on;
    switch ii
        case 1; 
            xlabel('glucose cycle');
            title('(a)');
        case 2; xlabel('fructose cycle');
            title('(b)');
        case 3; xlabel('mixed cycle');
            title('(c)');
    end
end

%% load data
% idxCycle = 1 ... glucose cycle
% idxCycle = 2 ... fructose cycle
% idxCycle = 3 ... mixed cycle

for idxCycle=1:3
    
    switch idxCycle
        case 1; hs(4) = subplot(2,3,4);
        case 2; hs(5) = subplot(2,3,5);
        case 3; hs(6) = subplot(2,3,6);
    end
    hp = plot(NaN,NaN); hp = repmat(hp,11,1);
    ymin = 0.225; ymax = 0.425;
    vert = [wn1-waveband ymin; wn1+waveband ymin;
            wn1+waveband ymax; wn1-waveband ymax;
            wn2-waveband ymin; wn2+waveband ymin;
            wn2+waveband ymax; wn2-waveband ymax
            wn3-waveband ymin; wn3+waveband ymin;
            wn3+waveband ymax; wn3-waveband ymax
            wn4-waveband ymin; wn4+waveband ymin;
            wn4+waveband ymax; wn4-waveband ymax];
    switch idxCycle
        case 1;
            fac  = [1 2 3 4; 9 10 11 12];
            fclr = lg;
            hb1 = patch('Faces',fac,'Vertices',vert,...
                        'FaceColor',fclr,'FaceAlpha',0.4,...
                        'EdgeColor','none');
        case 2;
            fac  = [5 6 7 8; 13 14 15 16];
            fclr = lbCD;
            hb1 = patch('Faces',fac,'Vertices',vert,...
                        'FaceColor',fclr,'FaceAlpha',0.4,...
                        'EdgeColor','none');
        case 3
            fac1  = [1 2 3 4;  9 10 11 12];
            fac2  = [5 6 7 8; 13 14 15 16];
            fclr1 = lg;
            fclr2 = lbCD;
            hb2 = patch('Faces',fac2,'Vertices',vert,...
                        'FaceColor',fclr2,'FaceAlpha',0.4,...
                        'EdgeColor','none');
            hb1 = patch('Faces',fac1,'Vertices',vert,...
                        'FaceColor',fclr1,'FaceAlpha',0.4,...
                        'EdgeColor','none');
    end

    switch idxCycle
        case 1;
            matFile = 'gc_data.mat';
            datasets = {'gc_1_c10_gl';'gc_1_water';...
                        'gc_2_c25_gl';'gc_2_water';...
                        'gc_3_c50_gl';'gc_3_water';...
                        'gc_4_c25_gl';'gc_4_water';...
                        'gc_5_c10_gl';'gc_5_water';...
                        'gc_6_water'};
        case 2;
            matFile = 'fc_data.mat';
            datasets = {'fc_1_c10_gl';'fc_1_water';...
                        'fc_2_c25_gl';'fc_2_water';...
                        'fc_3_c50_gl';'fc_3_water';...
                        'fc_4_c25_gl';'fc_4_water';...
                        'fc_5_c10_gl';'fc_5_water';...
                        'fc_6_water'};
        case 3;
            matFile = 'mc_data.mat';
            datasets = {'mc_1_G50_F0';'mc_1_water';...
                        'mc_2_F50_G0';'mc_2_water';...
                        'mc_3_F50_G50';'mc_3_water';...
                        'mc_4_30F_60G';'mc_4_water';...
                        'mc_5_F60_G30';'mc_5_water';...
                        'mc_6_water'};
    end
    load(matFile);

    counter_clr = 0;
    for idxSet=[2,4,6,8,10,11,1,9,3,7,5]
        
        counter_clr = counter_clr+1;

        % data set
        dataset = datasets{idxSet};
        eval(['x =',dataset,'.data1;']) 
        eval(['y =',dataset,'.data2;']) 

        % convert cell to array
        n = length(x{1});               % number of measurements
        m = length(x);                  % number of sampled data points
        x = reshape(cell2mat(x),n,m);   % wavenumbers [cm-1]
        y = reshape(cell2mat(y),n,m);   % rel. reflectance
    
        % mean value
        xc = mean(x,2);
        yc = mean(y,2);
            
        % plot
        hold on;
        switch idxCycle
            case 1; hp(counter_clr) = plot(xc,yc,cls,clr1(counter_clr,:),lws,lw);
            case 2; hp(counter_clr) = plot(xc,yc,cls,clr2(counter_clr,:),lws,lw);
            case 3; hp(counter_clr) = plot(xc,yc,cls,clr3(counter_clr,:),lws,lw);
        end
    end
    
    % additional settings for figure
    xlabel('\nu [cm^{-1}]')
    xlim([1000 1100]);
    set(gca,'XTick',[1000 1025 1050 1075 1100]);
%     set(gca,'XTickLabel',{'1000','','1050','','1100'});
    if idxCycle==1
        ylabel('r(\nu)');
        set(gca,'YTick',[0.25 0.30 0.35 0.40]);
        set(gca,'YTickLabel',{'0.25','0.30','0.35','0.40'});
    else
        ylabel(' ');
        set(gca,'YTickLabel',{});
    end
    ylim([ymin ymax]);
    grid on; box on;
    locationstr = 'south';%'northoutside';
    if idxCycle==1
        hl1 = legend([hp(1), hp(7), hp(9), hp(11)],...
           {'water','c_{glucose} = 10 g/l','c_{glucose} = 25 g/l','c_{glucose} = 50 g/l'},...
           'location',locationstr,fss,fs_legend,...
           'orientation','vertical');
       title('(d)');
    elseif idxCycle==2
        hl2 = legend([hp(1), hp(7), hp(9), hp(11)],...
               {'water','c_{fructose} = 10 g/l','c_{fructose} = 25 g/l','c_{fructose} = 50 g/l'},...
               'location',locationstr,fss,fs_legend);
        title('(e)');
    else
        hl3 = legend([hp(7), hp(9), hp(11), hp(10), hp(8)],...
               {'c_g:c_f = 50:0 g/l',...
                'c_g:c_f = 0:50 g/l',...
                'c_g:c_f = 50:50 g/l',...
                'c_g:c_f = 60:30 g/l',...
                'c_g:c_f = 30:60 g/l'},...
                'location',locationstr,fss,fs_legend);
        title('(f)');
    end
end

set(hs(1),'OuterPosition',[  0, 1/2, 1/3, 1/2]);
set(hs(2),'OuterPosition',[1/3, 1/2, 1/3, 1/2]);
set(hs(3),'OuterPosition',[2/3, 1/2, 1/3, 1/2]);
set(hs(4),'OuterPosition',[  0, 0,   1/3, 1/2]);
set(hs(5),'OuterPosition',[1/3, 0,   1/3, 1/2]);
set(hs(6),'OuterPosition',[2/3, 0,   1/3, 1/2]);

% Adjust font size
set(findobj(gcf,fss,10),fss,fs)

% export figure
cd('..\02_Graphics')
orient(hf,'landscape')
set(hf,'PaperSize',[29.7 21])
set(hf,'PaperPosition',[0 0 29.7 21])
set(hf,'Renderer','painters')
print(hf,'figure_3_paper','-djpeg','-r300');
print(hf,'figure_3_paper','-dpdf', '-r300');
cd(cfroot)

