%% Generate figure 2 - sensitivity of ALSS for gc 50g/l
%
% Author:           Benjamin Schuler
% Company:          University of Stuttgart, Institute for System Dynamics
% Last modified:    21.02.2019 (Matlab 2015a)

clear; close all; clc;
cfroot = cd;

%% settings

% window for wavenumbers
w_exc_LB = 1000;        % lower bound for exclusion from ALSS
w_exc_UB = 1100;        % upper bound for exclusion from ALSS
waveband = 3;           % band of wavenumbers around characteristic wavenumbers nu_i
w_LB = 800;             % lower bound of wavenumber-sepctrum for plot
w_UB = 2500;            % upper bound of wavenumber-sepctrum for plot

% parameters for Asymmetric Least Squares Smoothing (ALSS)
lambda = logspace(-3,3,7);
p      = 0.99;

% characteristic wavenumbers
wn1 = 1034;    % [1/cm]    % 1. characteristic wavenumber of glucose
wn2 = 1063;    % [1/cm]    % 1. characteristic wavenumber of fructose
wn3 = 1078;    % [1/cm]    % 2. characteristic wavenumber of glucose/fructose

% load plot style
load('plotstyleBS.mat')
fs = 14; % font size
fs_legend = 13; % font size in legend
clr = [linspace(lg(1),dg(1),length(lambda))',...
       linspace(lg(2),dg(2),length(lambda))',...
       linspace(lg(3),dg(3),length(lambda))'];    % color vector

%% load data
% idxCycle = 1 ... glucose cycle
% idxCycle = 2 ... fructose cycle
% idxCycle = 3 ... mixed cycle

for idxCycle=1
    
    switch idxCycle
        case 1;
            matFile = 'gc_data.mat';
            datasets = {'gc_1_c10_gl';'gc_1_water';...
                        'gc_2_c25_gl';'gc_2_water';...
                        'gc_3_c50_gl';'gc_3_water';...
                        'gc_4_c25_gl';'gc_4_water';...
                        'gc_5_c10_gl';'gc_5_water';...
                        'gc_6_water'};
        case 2;
            matFile = 'fc_data.mat';
            datasets = {'fc_1_c10_gl';'fc_1_water';...
                        'fc_2_c25_gl';'fc_2_water';...
                        'fc_3_c50_gl';'fc_3_water';...
                        'fc_4_c25_gl';'fc_4_water';...
                        'fc_5_c10_gl';'fc_5_water';...
                        'fc_6_water'};
        case 3;
            matFile = 'mc_data.mat';
            datasets = {'mc_1_G50_F0';'mc_1_water';...
                        'mc_2_F50_G0';'mc_2_water';...
                        'mc_3_F50_G50';'mc_3_water';...
                        'mc_4_30F_60G';'mc_4_water';...
                        'mc_5_F60_G30';'mc_5_water';...
                        'mc_6_water'};
    end
    load(matFile);

    for idxSet=5

        % data set
        dataset = datasets{idxSet};
        eval(['x =',dataset,'.data1;']) 
        eval(['y =',dataset,'.data2;']) 

        % convert cell to array
        n = length(x{1});               % number of measurements
        m = length(x);                  % number of sampled data points
        x = reshape(cell2mat(x),n,m);   % wavenumbers [cm-1]
        y = reshape(cell2mat(y),n,m);   % rel. reflectance
    
        % mean value
        xc = mean(x,2);
        yc = mean(y,2);
       
        % apply ALSS
        w_a = ones(size(xc));
        idxLB = find(xc>w_exc_LB,1,'first');
        idxUB = find(xc>w_exc_UB,1,'first');
        w_a(idxLB:idxUB) = zeros(idxUB-idxLB+1,1);
        zc    = cell(length(lambda),length(p));
        zc_bc = cell(length(lambda),length(p));
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                zc{idxL,idxP}    = fcn_alss(yc,w_a,lambda(idxL),p(idxP));
                zc_bc{idxL,idxP} = yc./zc{idxL,idxP};
            end
        end
        
        % extract height of peak at characteristic wavenumbers
        [~,idx1] = min((xc-wn1).^2);
        [~,idx2] = min((xc-wn2).^2);
        [~,idx3] = min((xc-wn3).^2);
        s1 = zeros(length(lambda),1);
        s2 = s1; s3 = s1;
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                s1(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx1-waveband):(idx1+waveband))))*100;
                s2(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx2-waveband):(idx2+waveband))))*100;
                s3(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx3-waveband):(idx3+waveband))))*100;
            end
        end
               
        % plot
        idx_wLB = find(xc>=1000,1,'first');     % Index LB
        idx_wUB = find(xc>=1100,1,'first')-1;   % Index UB
        ymin  = 0.00; ymax  = 0.50;
        yminz = 0.34; ymaxz = 0.42;
        ymin_bc  = 0.80; ymax_bc  = 1.20;
        yminz_bc = 0.95; ymaxz_bc = 1.05;
        
        % initialize figure
        hf = figure('color','white','visible','off');

        hs1 = subplot(2,3,[1 2]);
        vertArea = [w_LB ymin; w_exc_LB ymin; 
                    w_exc_LB ymax; w_LB ymax;
                    w_exc_UB ymin; w_UB ymin; 
                    w_UB ymax; w_exc_UB ymax];
        faceArea = [1 2 3 4; 5 6 7 8];
        vertAreaZ = [w_exc_LB yminz; w_exc_UB yminz; 
                     w_exc_UB ymaxz; w_exc_LB ymaxz];
        hA = patch('Faces',faceArea,'Vertices',vertArea,...
             'FaceColor',lgrCD,'FaceAlpha',0.5,'EdgeColor','none'); hold on;
        hAZ = patch('Faces',faceArea(1,:),'Vertices',vertAreaZ,...
             'FaceColor','none','FaceAlpha',1,'EdgeColor',dgrCD);
        hp1 = plot(xc,yc,cls,dgrCD,lws,2); hold on;
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                hp2 = plot(xc,zc{idxL,idxP},cls,clr(idxL,:));
            end
        end
        title('(a)');
        legend([hp1 hp2],...
               {'reflectance spectrum (50 g/l glucose)','set of baselines'},...
                'location','northeast',fss,fs_legend);

        hs2 = subplot(2,3,3);
        vert = [wn1-waveband yminz; wn1+waveband yminz;
                wn1+waveband ymaxz; wn1-waveband ymaxz;
                wn2-waveband yminz; wn2+waveband yminz;
                wn2+waveband ymaxz; wn2-waveband ymaxz;
                wn3-waveband yminz; wn3+waveband yminz;
                wn3+waveband ymaxz; wn3-waveband ymaxz];
        fac1 = [1 2 3 4]; fclr1 = lg;
        fac2 = [5 6 7 8]; fclr2 = lbCD;
        fac3 = [9 10 11 12]; fclr3 = (lg+lbCD)/2;
        hb1 = patch('Faces',fac1,'Vertices',vert,'FaceColor',fclr1,'FaceAlpha',0.4,'EdgeColor','none'); hold on;
        hb2 = patch('Faces',fac2,'Vertices',vert,'FaceColor',fclr2,'FaceAlpha',0.4,'EdgeColor','none');
        hb3 = patch('Faces',fac3,'Vertices',vert,'FaceColor',fclr3,'FaceAlpha',0.4,'EdgeColor','none');
        hp1 = plot(xc,yc,cls,dgrCD,lws,2);
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                hp2 = plot(xc,zc{idxL,idxP},cls,clr(idxL,:)); hold on;
            end
        end
        title('(b)');

        hs3 = subplot(2,3,[4 5]);
        vertArea = [w_LB ymin_bc; w_exc_LB ymin_bc; 
                    w_exc_LB ymax_bc; w_LB ymax_bc;
                    w_exc_UB ymin_bc; w_UB ymin_bc; 
                    w_UB ymax_bc; w_exc_UB ymax_bc];
        faceArea = [1 2 3 4; 5 6 7 8];
        vertAreaZ = [w_exc_LB yminz_bc; w_exc_UB yminz_bc;...
                     w_exc_UB ymaxz_bc; w_exc_LB ymaxz_bc];
        hA = patch('Faces',faceArea,'Vertices',vertArea,...
             'FaceColor',lgrCD,'FaceAlpha',0.5,'EdgeColor','none'); hold all;
        hAZ = patch('Faces',faceArea(1,:),'Vertices',vertAreaZ,...
             'FaceColor','none','FaceAlpha',1,'EdgeColor',dgrCD);
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                hp2 = plot(xc,zc_bc{idxL,idxP},cls,clr(idxL,:),lws,1);
            end
        end
        title('(c)');

        hs4 = subplot(2,3,6);
        vert = [wn1-waveband yminz_bc; wn1+waveband yminz_bc;
                wn1+waveband ymaxz_bc; wn1-waveband ymaxz_bc;
                wn2-waveband yminz_bc; wn2+waveband yminz_bc;
                wn2+waveband ymaxz_bc; wn2-waveband ymaxz_bc;
                wn3-waveband yminz_bc; wn3+waveband yminz_bc;
                wn3+waveband ymaxz_bc; wn3-waveband ymaxz_bc];
        fac1 = [1 2 3 4]; fclr1 = lg;
        fac2 = [5 6 7 8]; fclr2 = lbCD;
        fac3 = [9 10 11 12]; fclr3 = (lg+lbCD)/2;
        hb1 = patch('Faces',fac1,'Vertices',vert,'FaceColor',fclr1,'FaceAlpha',0.4,'EdgeColor','none'); hold on;
        hb2 = patch('Faces',fac2,'Vertices',vert,'FaceColor',fclr2,'FaceAlpha',0.4,'EdgeColor','none');
        hb3 = patch('Faces',fac3,'Vertices',vert,'FaceColor',fclr3,'FaceAlpha',0.4,'EdgeColor','none');
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                hp2 = plot(xc,zc_bc{idxL,idxP},cls,clr(idxL,:),lws,1); hold all;
            end
        end
        title('(d)');
        
        set(hs1,'OuterPosition',[  0, 0.5, 2/3, 0.5]);
        set(hs2,'OuterPosition',[2/3, 0.5, 1/3, 0.5],'XColor',dgrCD,'YColor',dgrCD);
        set(hs3,'OuterPosition',[  0, 0.0, 2/3, 0.5]);
        set(hs4,'OuterPosition',[2/3, 0.0, 1/3, 0.5],'XColor',dgrCD,'YColor',dgrCD);
        
        set(get(hs1,'XLabel'),'String','\nu [cm^{-1}]');
        set(get(hs2,'XLabel'),'String','\nu [cm^{-1}]');
        set(get(hs3,'XLabel'),'String','\nu [cm^{-1}]');
        set(get(hs4,'XLabel'),'String','\nu [cm^{-1}]');
        
        set(hs2,'YAxisLocation','right');
        set(hs4,'YAxisLocation','right');
        
        set(get(hs1,'YLabel'),'String','r(\nu)');
        set(get(hs2,'YLabel'),'String','r(\nu)');
        set(get(hs3,'YLabel'),'String','r_{BC}(\nu)');
        set(get(hs4,'YLabel'),'String','r_{BC}(\nu)');
        
        set(hs1,'XGrid','On','YGrid','On','Box','On');
        set(hs2,'XGrid','On','YGrid','On','Box','On');
        set(hs3,'XGrid','On','YGrid','On','Box','On');
        set(hs4,'XGrid','On','YGrid','On','Box','On');
        
        set(hs1,'XLim',[w_LB w_UB],'YLim',[ymin ymax]);
        set(hs2,'XLim',[w_exc_LB w_exc_UB],'YLim',[yminz ymaxz]);
        set(hs3,'XLim',[w_LB w_UB],'YLim',[ymin_bc ymax_bc]);
        set(hs4,'XLim',[w_exc_LB w_exc_UB],'YLim',[yminz_bc ymaxz_bc]);

        % adjust font size
        set(findobj(gcf,fss,10),fss,fs)

        % export figure
        cd('..\02_Graphics')
        orient(hf,'landscape')
        set(hf,'PaperSize',[29.7 21])
        set(hf,'PaperPosition',[0 0 29.7 21])
        set(hf,'Renderer','painters')
        print(hf,'figure_4_paper','-djpeg','-r300');
        print(hf,'figure_4_paper','-dpdf', '-r300');
        cd(cfroot)
    end
end