%% Preprocess data
% Results are saved to results_alss_bc.xlsx
%
% Author:           Benjamin Schuler
% Company:          University of Stuttgart, Institute for System Dynamics
% Last modified:    21.02.2019 (Matlab 2015a)

clear; close all; clc;
cfroot = cd;

%% settings

% window for wavenumbers
bWindowOn = true;
w_exc_LB = 1000;       % lower bound for exclusion from ALSS
w_exc_UB = 1100;       % upper bound for exclusion from ALSS
waveband = 3;          % band of wavenumbers around characteristic 
                       % wavenumbers wn_i

% parameters for Asymmetric Least Squares Smoothing (ALSS)
lambda = 500;
p      = 0.99;

% characteristic wavenumbers
wn1 = 1034;    % [1/cm]    % 1. characteristic wavenumber of glucose
wn2 = 1063;    % [1/cm]    % 1. characteristic wavenumber of fructose
wn3 = 1078;    % [1/cm]    % 2. characteristic wavenumber of glucose/fructose  

%% load SEIRA-data
% idxCycle = 1 ... glucose cycle
% idxCycle = 2 ... fructose cycle
% idxCycle = 3 ... mixed cycle

for idxCycle=1:3
    switch idxCycle
        case 1;
            matFile = 'gc_data.mat';
            datasets = {'gc_1_c10_gl';'gc_1_water';...
                        'gc_2_c25_gl';'gc_2_water';...
                        'gc_3_c50_gl';'gc_3_water';...
                        'gc_4_c25_gl';'gc_4_water';...
                        'gc_5_c10_gl';'gc_5_water';...
                        'gc_6_water'};
        case 2;
            matFile = 'fc_data.mat';
            datasets = {'fc_1_c10_gl';'fc_1_water';...
                        'fc_2_c25_gl';'fc_2_water';...
                        'fc_3_c50_gl';'fc_3_water';...
                        'fc_4_c25_gl';'fc_4_water';...
                        'fc_5_c10_gl';'fc_5_water';...
                        'fc_6_water'};
        case 3;
            matFile = 'mc_data.mat';
            datasets = {'mc_1_G50_F0';'mc_1_water';...
                        'mc_2_F50_G0';'mc_2_water';...
                        'mc_3_F50_G50';'mc_3_water';...
                        'mc_4_30F_60G';'mc_4_water';...
                        'mc_5_F60_G30';'mc_5_water';...
                        'mc_6_water'};
    end
    load(matFile);

    for idxSet=1:length(datasets)

        % data set
        dataset = datasets{idxSet};
        eval(['x =',dataset,'.data1;']) 
        eval(['y =',dataset,'.data2;']) 

        % convert cell to array
        n = length(x{1});               % number of measurements
        m = length(x);                  % number of sampled data points
        x = reshape(cell2mat(x),n,m);   % wavenumbers [cm-1]
        y = reshape(cell2mat(y),n,m);   % rel. reflectance
    
        % mean value
        xc = mean(x,2);
        yc = mean(y,2);
       
        % apply ALSS
        w_a = ones(size(xc));
        if bWindowOn==true
            idxLB = find(xc>w_exc_LB,1,'first');
            idxUB = find(xc>w_exc_UB,1,'first');
            w_a(idxLB:idxUB) = zeros(idxUB-idxLB+1,1);
        end
        zc    = cell(length(lambda),length(p));
        zc_bc = cell(length(lambda),length(p));
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                zc{idxL,idxP}    = fcn_alss(yc,w_a,lambda(idxL),p(idxP));
                zc_bc{idxL,idxP} = yc./zc{idxL,idxP};
            end
        end
        
        % extract height of peak at characteristic wavenumbers
        [~,idx1] = min((xc-wn1).^2);
        [~,idx2] = min((xc-wn2).^2);
        [~,idx3] = min((xc-wn3).^2);
        s1 = zeros(length(lambda),1);
        s2 = s1; s3 = s1;
        for idxL=1:length(lambda)
            for idxP=1:length(p)
                s1(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx1-waveband):(idx1+waveband))))*100;
                s2(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx2-waveband):(idx2+waveband))))*100;
                s3(idxL,idxP) = (1-min(zc_bc{idxL,idxP}((idx3-waveband):(idx3+waveband))))*100;
            end
        end
        
        % export results to xlsx-file
        file   = 'results_alss_bc.xlsx';
        if bWindowOn==true
            sheet = 'results_pp';       % results pre-processing
        else
            sheet = 'results_pp_nw';    % results pre-processing with no window
        end
        xlData = [p,lambda,s1,s2,s3];
        switch idxCycle
            case 1; idxOffset = 1;
            case 2; idxOffset = 12;
            case 3; idxOffset = 23;
        end
        xlRange = ['B',num2str(idxSet+idxOffset),...
                   ':F',num2str(idxSet+idxOffset)];
        [status,message] = xlswrite(file,xlData,sheet,xlRange);
        disp(['data set: ',num2str(idxSet,'% 2.0f'),...
              '   status xls: ',num2str(status)]);
    end
end
