%% Check for best parameter configuration
% Results are saved to results_ddp.xlsx
%
% Author:           Benjamin Schuler
% Company:          University of Stuttgart, Institute for System Dynamics
% Last modified:    21.02.2019 (Matlab 2015a)

clear; close all; clc;
cfroot = cd;

%% settings

% parameters for Asymmetric Least Squares Smoothing (ALSS)
lambda = [500 750 1000 1250 1500];%[0.001 0.1 1 10 100 1000 5000 10000];
p      = [0.01, 0.25, 0.5, 0.75, 0.99];

% characteristic wavenumbers
wn1 = 1034;    % [1/cm]    % 1. characteristic wavenumber of glucose
wn2 = 1063;    % [1/cm]    % 1. characteristic wavenumber of fructose
wn3 = 1078;    % [1/cm]    % 2. characteristic wavenumber of glucose/fructose

% window for wavenumbers
w_exc = [1000, 1100];  % lower/upper bound for exclusion from ALSS
% w_exc = [wn1-25, wn3+25;...
%          wn1-30, wn3+30;...
%          wn1-35, wn3+35];
waveband = 3;          % band of wavenumbers around characteristic 
                       % wavenumbers wn_i
ng = 1:3;   % polynomial degree for phi_i
nf = 1:3;   % polynomial degree for psi_i

% options for fitting of basic functions
addzeros = 10;  % weighting for origin

% excel file
file      = 'results_ddp.xlsx';
sheet_est = 'results_est';          % results concentration estimate

xlData = zeros(size(w_exc,1)*length(lambda)*length(p)*length(ng)*length(nf),18);
cntr = 0;

for idxW = 1:size(w_exc,1)
    for idxL = 1:length(lambda)
        idxL
        tic
        for idxP = 1:length(p)
            for idxNG = 1:length(ng)
                for idxNF = 1:length(nf)
                    
                    %% pre-process and analyze SEIRA-data
                    % idxCycle = 1 ... glucose cycle
                    % idxCycle = 2 ... fructose cycle
                    % idxCycle = 3 ... mixed cycle
                    
                    ds1034 = struct('gc',zeros(11,1),'fc',zeros(11,1),'mc',zeros(11,1)); 
                    ds1063 = struct('gc',zeros(11,1),'fc',zeros(11,1),'mc',zeros(11,1));
                    ds1078 = struct('gc',zeros(11,1),'fc',zeros(11,1),'mc',zeros(11,1));
                    
                    for idxCycle=1:3
                        switch idxCycle
                            case 1;
                                matFile = 'gc_data.mat';
                                datasets = {'gc_1_c10_gl';'gc_1_water';...
                                            'gc_2_c25_gl';'gc_2_water';...
                                            'gc_3_c50_gl';'gc_3_water';...
                                            'gc_4_c25_gl';'gc_4_water';...
                                            'gc_5_c10_gl';'gc_5_water';...
                                            'gc_6_water'};
                            case 2;
                                matFile = 'fc_data.mat';
                                datasets = {'fc_1_c10_gl';'fc_1_water';...
                                            'fc_2_c25_gl';'fc_2_water';...
                                            'fc_3_c50_gl';'fc_3_water';...
                                            'fc_4_c25_gl';'fc_4_water';...
                                            'fc_5_c10_gl';'fc_5_water';...
                                            'fc_6_water'};
                            case 3;
                                matFile = 'mc_data.mat';
                                datasets = {'mc_1_G50_F0';'mc_1_water';...
                                            'mc_2_F50_G0';'mc_2_water';...
                                            'mc_3_F50_G50';'mc_3_water';...
                                            'mc_4_30F_60G';'mc_4_water';...
                                            'mc_5_F60_G30';'mc_5_water';...
                                            'mc_6_water'};
                        end
                        load(matFile);

                        for idxSet=1:length(datasets)

                            % data set
                            dataset = datasets{idxSet};
                            eval(['x =',dataset,'.data1;']) 
                            eval(['y =',dataset,'.data2;']) 

                            % convert cell to array
                            n = length(x{1});               % number of measurements
                            m = length(x);                  % number of sampled data points
                            x = reshape(cell2mat(x),n,m);   % wavenumbers [cm-1]
                            y = reshape(cell2mat(y),n,m);   % rel. reflectance

                            % mean value
                            xc = mean(x,2);
                            yc = mean(y,2);

                            % apply ALSS
                            w_a = ones(size(xc));
                            idxLB = find(xc>w_exc(idxW,1),1,'first');
                            idxUB = find(xc>w_exc(idxW,2),1,'first');
                            w_a(idxLB:idxUB) = zeros(idxUB-idxLB+1,1);
                            zc    = fcn_alss(yc,w_a,lambda(idxL),p(idxP));
                            zc_bc = yc./zc;

                            % extract height of peak at characteristic wavenumbers
                            [~,idx1] = min((xc-wn1).^2);
                            [~,idx2] = min((xc-wn2).^2);
                            [~,idx3] = min((xc-wn3).^2);
                            s1 = (1-min(zc_bc((idx1-waveband):(idx1+waveband))))*100;
                            s2 = (1-min(zc_bc((idx2-waveband):(idx2+waveband))))*100;
                            s3 = (1-min(zc_bc((idx3-waveband):(idx3+waveband))))*100;
                            
                            switch idxCycle
                                case 1;
                                    ds1034.gc(idxSet,1) = s1;
                                    ds1063.gc(idxSet,1) = s2;
                                    ds1078.gc(idxSet,1) = s3;
                                case 2;
                                    ds1034.fc(idxSet,1) = s1;
                                    ds1063.fc(idxSet,1) = s2;
                                    ds1078.fc(idxSet,1) = s3;
                                case 3;
                                    ds1034.mc(idxSet,1) = s1;
                                    ds1063.mc(idxSet,1) = s2;
                                    ds1078.mc(idxSet,1) = s3;
                            end
                        end
                    end

                    %% Concentration estimate
                    
                    nFit_g = ng(idxNG);     % nFit_g ... polynomial order for glucose
                    nFit_f = nf(idxNF);     % nFit_f ... polynomial order for fructose
                    
                    % sort
                    idxSort = [2,1,4,3,6,5,8,7,10,9,11];
                    ds1034.gc = ds1034.gc(idxSort);
                    ds1034.fc = ds1034.fc(idxSort);
                    ds1034.mc = ds1034.mc(idxSort);
                    ds1063.gc = ds1063.gc(idxSort);
                    ds1063.fc = ds1063.fc(idxSort);
                    ds1063.mc = ds1063.mc(idxSort);
                    ds1078.gc = ds1078.gc(idxSort);
                    ds1078.fc = ds1078.fc(idxSort);
                    ds1078.mc = ds1078.mc(idxSort);

                    % pure water
                    idx_w = 1:2:11;
                    ds1034_w_gc = mean(ds1034.gc(idx_w,1));
                    ds1034_w_fc = mean(ds1034.fc(idx_w,1));
                    ds1034_w_mc = mean(ds1034.mc(idx_w,1));
                    ds1063_w_gc = mean(ds1063.gc(idx_w,1));
                    ds1063_w_fc = mean(ds1063.fc(idx_w,1));
                    ds1063_w_mc = mean(ds1063.mc(idx_w,1));
                    ds1078_w_gc = mean(ds1078.gc(idx_w,1));
                    ds1078_w_fc = mean(ds1078.fc(idx_w,1));
                    ds1078_w_mc = mean(ds1078.mc(idx_w,1));
                    ds1034_w = [(ds1034.gc(idx_w,1)-ds1034_w_gc);...
                                (ds1034.fc(idx_w,1)-ds1034_w_fc);...
                                (ds1034.mc(idx_w,1)-ds1034_w_mc)];
                    ds1063_w = [(ds1063.gc(idx_w,1)-ds1063_w_gc);...
                                (ds1063.fc(idx_w,1)-ds1063_w_fc);...
                                (ds1063.mc(idx_w,1)-ds1063_w_mc)];
                    ds1078_w = [(ds1078.gc(idx_w,1)-ds1078_w_gc);...
                                (ds1078.fc(idx_w,1)-ds1078_w_fc);...
                                (ds1078.mc(idx_w,1)-ds1078_w_mc)];
                            
                    % pure glucose (with subtractet water)
                    idx_c = [10,10,25,25,50,50]';
                    ds1034_g = [ds1034.gc(2)-ds1034_w_gc;...
                                ds1034.gc(10)-ds1034_w_gc;...
                                ds1034.gc(4)-ds1034_w_gc;...
                                ds1034.gc(8)-ds1034_w_gc;...
                                ds1034.gc(6)-ds1034_w_gc;...
                                ds1034.mc(2)-ds1034_w_mc;...
                                ];
                    ds1063_g = [ds1063.gc(2)-ds1063_w_gc;...
                                ds1063.gc(10)-ds1063_w_gc;...
                                ds1063.gc(4)-ds1063_w_gc;...
                                ds1063.gc(8)-ds1063_w_gc;...
                                ds1063.gc(6)-ds1063_w_gc;...
                                ds1063.mc(2)-ds1063_w_mc;...
                                ];
                    ds1078_g = [ds1078.gc(2)-ds1078_w_gc;...
                                ds1078.gc(10)-ds1078_w_gc;...
                                ds1078.gc(4)-ds1078_w_gc;...
                                ds1078.gc(8)-ds1078_w_gc;...
                                ds1078.gc(6)-ds1078_w_gc;...
                                ds1078.mc(2)-ds1078_w_mc;...
                                ];
                            
                    % pure fructose (with subtractet water)
                    ds1034_f = [ds1034.fc(2)-ds1034_w_fc;...
                                ds1034.fc(10)-ds1034_w_fc;...
                                ds1034.fc(4)-ds1034_w_fc;...
                                ds1034.fc(8)-ds1034_w_fc;...
                                ds1034.fc(6)-ds1034_w_fc;...
                                ds1034.mc(4)-ds1034_w_mc;...
                                ];
                    ds1063_f = [ds1063.fc(2)-ds1063_w_fc;...
                                ds1063.fc(10)-ds1063_w_fc;...
                                ds1063.fc(4)-ds1063_w_fc;...
                                ds1063.fc(8)-ds1063_w_fc;...
                                ds1063.fc(6)-ds1063_w_fc;...
                                ds1063.mc(4)-ds1063_w_mc;...
                                ];
                    ds1078_f = [ds1078.fc(2)-ds1078_w_fc;...
                                ds1078.fc(10)-ds1078_w_fc;...
                                ds1078.fc(4)-ds1078_w_fc;...
                                ds1078.fc(8)-ds1078_w_fc;...
                                ds1078.fc(6)-ds1078_w_fc;...
                                ds1078.mc(4)-ds1078_w_mc;...
                                ];
        
                    coeff_ds1034_g_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1034_g;zeros(addzeros,1)],nFit_g);
                    ds1034_g_fit       = polyval(coeff_ds1034_g_fit,0:50);
                    coeff_ds1063_g_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1063_g;zeros(addzeros,1)],nFit_g);
                    ds1063_g_fit       = polyval(coeff_ds1063_g_fit,0:50);
                    coeff_ds1078_g_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1078_g;zeros(addzeros,1)],nFit_g);
                    ds1078_g_fit       = polyval(coeff_ds1078_g_fit,0:50);

                    coeff_ds1034_f_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1034_f;zeros(addzeros,1)],nFit_f);
                    ds1034_f_fit       = polyval(coeff_ds1034_f_fit,0:50);
                    coeff_ds1063_f_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1063_f;zeros(addzeros,1)],nFit_f);
                    ds1063_f_fit       = polyval(coeff_ds1063_f_fit,0:50);
                    coeff_ds1078_f_fit = polyfit([idx_c;zeros(addzeros,1)],[ds1078_f;zeros(addzeros,1)],nFit_f);
                    ds1078_f_fit       = polyval(coeff_ds1078_f_fit,0:50);
        
                    % reconstruction of concentrations
                    x0 = [0;0];
                    options = optimset('Algorithm',{'levenberg-marquardt',0.005},...
                                       'Display', 'off',...% 'iter' / 'off'
                                       'TolFun', 1e-6,...%1e-12,...
                                       'TolX', 1e-6,...%1e-12,...
                                       'MaxIter', 1e3,...%1e8,...
                                       'MaxFunEvals',1e3);%1e8);%'PlotFcns', {  @optimplotx @optimplotfval });
                    param.nFit_g = nFit_g;
                    param.nFit_f = nFit_f;
                    switch nFit_g
                        case 1;
                            param.a = [0, 0, coeff_ds1034_g_fit];
                            param.b = [0, 0, coeff_ds1063_g_fit];
                            param.c = [0, 0, coeff_ds1078_g_fit];
                        case 2;
                            param.a = [0, coeff_ds1034_g_fit];
                            param.b = [0, coeff_ds1063_g_fit];
                            param.c = [0, coeff_ds1078_g_fit];
                        case 3;
                            param.a = coeff_ds1034_g_fit;
                            param.b = coeff_ds1063_g_fit;
                            param.c = coeff_ds1078_g_fit;
                        otherwise
                            warndlg('polynomial order to high. please choose between 1 and 3.')
                    end
                    switch nFit_f
                        case 1;
                            param.d = [0, 0, coeff_ds1034_f_fit];
                            param.e = [0, 0, coeff_ds1063_f_fit];
                            param.f = [0, 0, coeff_ds1078_f_fit];
                        case 2;
                            param.d = [0, coeff_ds1034_f_fit];
                            param.e = [0, coeff_ds1063_f_fit];
                            param.f = [0, coeff_ds1078_f_fit];
                        case 3;
                            param.d = coeff_ds1034_f_fit;
                            param.e = coeff_ds1063_f_fit;
                            param.f = coeff_ds1078_f_fit;
                        otherwise
                            warndlg('polynomial order to high. please choose between 1 and 3.')
                    end

                    % initialize concentration
                    cg_g = zeros(11,1);
                    cf_g = zeros(11,1);
                    cg_f = zeros(11,1);
                    cf_f = zeros(11,1);
                    cg_m = zeros(11,1);
                    cf_m = zeros(11,1);

                    % glucose cycle
                    ds_g = [(ds1034.gc-ds1034_w_gc)';
                            (ds1063.gc-ds1063_w_gc)';
                            (ds1078.gc-ds1078_w_gc)'];
                    for ii=1:length(cg_g)
                        param.ds = ds_g(:,ii);
                        [xOpt,fval,exitflag,output] = fsolve(@(x)fcn_polyfit(x,param),x0,options);
                        cg_g(ii,1) = xOpt(1);
                        cf_g(ii,1) = xOpt(2);
                    end

                    % fructose cycle
                    ds_f = [(ds1034.fc-ds1034_w_fc)';
                            (ds1063.fc-ds1063_w_fc)';
                            (ds1078.fc-ds1078_w_fc)'];
                    for ii=1:length(cg_f)
                        param.ds = ds_f(:,ii);
                        [xOpt,fval,exitflag,output] = fsolve(@(x)fcn_polyfit(x,param),x0,options);
                        cg_f(ii,1) = xOpt(1);
                        cf_f(ii,1) = xOpt(2);
                    end

                    % mixed cycle
                    ds_m = [(ds1034.mc-ds1034_w_mc)';
                            (ds1063.mc-ds1063_w_mc)';
                            (ds1078.mc-ds1078_w_mc)'];
                    for ii=1:length(cg_m)
                        param.ds = ds_m(:,ii);
                        [xOpt,fval,exitflag,output] = fsolve(@(x)fcn_polyfit(x,param),x0,options);
                        cg_m(ii,1) = xOpt(1);
                        cf_m(ii,1) = xOpt(2);
                    end

                    % set all negative concentrations to zero
                    cg_g(find(cg_g<0)) = zeros(size(find(cg_g<0)));
                    cf_g(find(cf_g<0)) = zeros(size(find(cf_g<0)));
                    cg_f(find(cg_f<0)) = zeros(size(find(cg_f<0)));
                    cf_f(find(cf_f<0)) = zeros(size(find(cf_f<0)));
                    cg_m(find(cg_m<0)) = zeros(size(find(cg_m<0)));
                    cf_m(find(cf_m<0)) = zeros(size(find(cf_m<0)));
        
                    % evaluate results
                    cg_gd = [0,10,0,25,0,50,0,25,0,10,0]';
                    cf_gd = zeros(11,1);
                    cg_fd = zeros(11,1);
                    cf_fd = [0,10,0,25,0,50,0,25,0,10,0]';
                    cg_md = [0,50,0, 0,0,50,0,60,0,30,0]';
                    cf_md = [0, 0,0,50,0,50,0,30,0,60,0]';

                    setvalue_vec = [cg_gd; cf_gd; cg_fd; cf_fd; cg_md; cf_md];
                    isvalue_vec  = [cg_g;  cf_g;  cg_f;  cf_f;  cg_m;  cf_m];
                    err_rms = rms(setvalue_vec-isvalue_vec);
                    err_avg = mean(abs(setvalue_vec-isvalue_vec));
                    err_max = max(abs(setvalue_vec-isvalue_vec));
                    err_rms_g = rms(setvalue_vec(1:22)-isvalue_vec(1:22));
                    err_avg_g = mean(abs(setvalue_vec(1:22)-isvalue_vec(1:22)));
                    err_max_g = max(abs(setvalue_vec(1:22)-isvalue_vec(1:22)));
                    err_rms_f = rms(setvalue_vec(23:44)-isvalue_vec(23:44));
                    err_avg_f = mean(abs(setvalue_vec(23:44)-isvalue_vec(23:44)));
                    err_max_f = max(abs(setvalue_vec(23:44)-isvalue_vec(23:44)));
                    err_rms_m = rms(setvalue_vec(45:66)-isvalue_vec(45:66));
                    err_avg_m = mean(abs(setvalue_vec(45:66)-isvalue_vec(45:66)));
                    err_max_m = max(abs(setvalue_vec(45:66)-isvalue_vec(45:66)));
                    
                    cntr = cntr+1;
                    xlData(cntr,:) = [w_exc(idxW,1),...
                                      w_exc(idxW,2),...
                                      lambda(idxL),...
                                      p(idxP),...
                                      idxNG,...
                                      idxNF,...
                                      err_rms,...
                                      err_avg,...
                                      err_max,...
                                      err_rms_g,...
                                      err_avg_g,...
                                      err_max_g,...
                                      err_rms_f,...
                                      err_avg_f,...
                                      err_max_f,...
                                      err_rms_m,...
                                      err_avg_m,...
                                      err_max_m];
%                     disp(['idxW: ',num2str(idxW),', ',...
%                           'idxL: ',num2str(idxL),', ',...
%                           'idxP: ',num2str(idxP),', ',...
%                           'idxG: ',num2str(idxNG),', ',...
%                           'idxF: ',num2str(idxNF)]);
                end
            end
        end
        toc
    end
end

% export results to xlsx-file
xlRange = ['A',num2str(3),...
           ':R',num2str(length(xlData)+2)];
[status,~] = xlswrite(file,xlData,sheet_est,xlRange);